==> charts-pie-1.0.0.tgz: package/lib/pie.js <==
/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

function __spreadArray(to, from) {
    for (var i = 0, il = from.length, j = to.length; i < il; i++, j++)
        to[j] = from[i];
    return to;
}

var ElementBuilder = /** @class */ (function () {
    function ElementBuilder(label) {
        this._elem = createElement(label);
    }
    ElementBuilder.prototype.setAttrs = function (attrs) {
        for (var _i = 0, _a = Object.entries(attrs); _i < _a.length; _i++) {
            var _b = _a[_i], key = _b[0], val = _b[1];
            this.attr(key, val);
        }
    };
    ElementBuilder.prototype.attr = function (name, val) {
        if (val) {
            setAttribute(this._elem, name, val);
            return this;
        }
        return getAttribute(this._elem, name) || '';
    };
    ElementBuilder.prototype.style = function (name, value) {
        this._elem.style.setProperty(name, value);
        return this;
    };
    Object.defineProperty(ElementBuilder.prototype, "element", {
        get: function () {
            return this._elem;
        },
        enumerable: false,
        configurable: true
    });
    ElementBuilder.prototype.append = function () {
        var _a;
        var elems = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            elems[_i] = arguments[_i];
        }
        (_a = this._elem).append.apply(_a, elems.map(function (elem) { return elem._elem; }));
        return this;
    };
    ElementBuilder.prototype.empty = function () {
        this._elem.innerHTML = '';
        return this;
    };
    ElementBuilder.create = function (label, attrs) {
        var instance = new ElementBuilder(label);
        if (attrs) {
            instance.setAttrs(attrs);
        }
        return instance;
    };
    return ElementBuilder;
}());
var SVG_NAMESPACE = 'http://www.w3.org/2000/svg';
function createElement(tag) {
    return document.createElementNS(SVG_NAMESPACE, tag);
}
function setAttribute(element, name, value) {
    if (name === 'xlink:href') {
        element.setAttributeNS(SVG_NAMESPACE, name, value);
    }
    element.setAttribute(name, value);
}
function getAttribute(element, name) {
    return element.getAttribute(name);
}

var random = function (min, max) { return min + Math.floor(Math.random() * (max - min)); };
var randomRgbColor = function () {
    var rgb = Array(3).fill(0).map(function () { return random(0, 256); });
    return "rgb(" + rgb.join(',') + ")";
};

var easeLinear = function (t) { return t; };
var easeIn = function (t, order) {
    if (order === void 0) { order = 2; }
    return Math.pow(t, order);
};

var Path = /** @class */ (function () {
    function Path() {
        this._pathSegments = [];
        this._isClosed = false;
    }
    Path.prototype.addPathSegments = function (instrument) {
        var params = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            params[_i - 1] = arguments[_i];
        }
        if (this._isClosed) {
            throw 'This path is already closed';
        }
        this._pathSegments.push(__spreadArray([instrument], params));
        if (instrument === 'Z') {
            this._isClosed = true;
        }
    };
    Path.prototype.moveTo = function (x, y) {
        this.addPathSegments('M', x, y);
        return this;
    };
    Path.prototype.moveToRelative = function (dx, dy) {
        this.addPathSegments('m', dx, dy);
        return this;
    };
    Path.prototype.lineTo = function (x, y) {
        this.addPathSegments('L', x, y);
        return this;
    };
    Path.prototype.lineToRelative = function (dx, dy) {
        this.addPathSegments('l', dx, dy);
        return this;
    };
    Path.prototype.horizontalLineTo = function (x) {
        this.addPathSegments('H', x);
        return this;
    };
    Path.prototype.horizontalLineToRelative = function (dx) {
        this.addPathSegments('h', dx);
        return this;
    };
    Path.prototype.verticalLineTo = function (y) {
        this.addPathSegments('V', y);
        return this;
    };
    Path.prototype.verticalLineToRelative = function (dy) {
        this.addPathSegments('v', dy);
        return this;
    };
    Path.prototype.curveTo = function (cx1, cy1, cx2, cy2, tx, ty) {
        this.addPathSegments('C', cx1, cy1, cx2, cy2, tx, ty);
        return this;
    };
    Path.prototype.curveToRelative = function (cdx1, cdy1, cdx2, cdy2, tdx, tdy) {
        this.addPathSegments('c', cdx1, cdy1, cdx2, cdy2, tdx, tdy);
        return this;
    };
    Path.prototype.smoothCurveTo = function (cx2, cy2, tx, ty) {
        this.addPathSegments('S', cx2, cy2, tx, ty);
        return this;
    };
    Path.prototype.smoothCurveToRelative = function (cdx2, cdy2, tdx, tdy) {
        this.addPathSegments('s', cdx2, cdy2, tdx, tdy);
        return this;
    };
    Path.prototype.quadCurveTo = function (cx, cy, tx, ty) {
        this.addPathSegments('Q', cx, cy, tx, ty);
        return this;
    };
    Path.prototype.quadCurveToRelative = function (cdx, cdy, tdx, tdy) {
        this.addPathSegments('q', cdx, cdy, tdx, tdy);
        return this;
    };
    Path.prototype.smoothQuadCurveTo = function (tx, ty) {
        this.addPathSegments('T', tx, ty);
        return this;
    };
    Path.prototype.smoothQuadCurveToRelative = function (tdx, tdy) {
        this.addPathSegments('t', tdx, tdy);
        return this;
    };
    Path.prototype.ellipticalArcTo = function (rx, ry, xAxisRotation, isLargeArc, isClockWise, tx, ty) {
        this.addPathSegments('A', rx, ry, xAxisRotation, isLargeArc ? 1 : 0, isClockWise ? 1 : 0, tx, ty);
        return this;
    };
    Path.prototype.closePath = function () {
        this.addPathSegments('Z');
        return this;
    };
    Path.prototype.back = function () {
        this._pathSegments.pop();
        return this;
    };
    Path.prototype.remove = function (index) {
        this._pathSegments.splice(index, 1);
        return this;
    };
    Path.prototype.toString = function () {
        return this.pathString;
    };
    Object.defineProperty(Path.prototype, "pathString", {
        get: function () {
            return this._pathSegments.map(function (_a) {
                var ins = _a[0], params = _a.slice(1);
                return ins + params.join(' ');
            }).join(' ');
        },
        enumerable: false,
        configurable: true
    });
    Path.prototype.append = function (path) {
        if (this._isClosed) {
            throw 'This path is already closed';
        }
        this._pathSegments = __spreadArray(__spreadArray([], this._pathSegments), path._pathSegments);
        if (this._pathSegments[this._pathSegments.length - 1][0] === 'Z') {
            this._isClosed = true;
        }
        return this;
    };
    return Path;
}());

var pointOnCircle = function (cx, cy, r, rad) {
    return [
        cx + r * Math.cos(rad),
        cy + r * Math.sin(rad)
    ];
};

var Tick = /** @class */ (function () {
    function Tick(cb) {
        this._startTime = 0;
        this._duration = 0;
        this._stopped = false;
        this._pending = false;
        this._tickCallback = cb;
    }
    Tick.prototype.start = function (duration) {
        if (this._pending)
            return;
        this._duration = duration;
        this._startTime = Date.now();
        this.tickLoop();
    };
    Tick.prototype.tickLoop = function () {
        var _this = this;
        this._pending = true;
        var tick = function () {
            var percentageComplete = (Date.now() - _this._startTime) / _this._duration;
            var over = false;
            if (percentageComplete > 1) {
                percentageComplete = 1;
                over = true;
            }
            if (!_this._stopped) {
                _this._tickCallback(percentageComplete);
            }
            if (!over && !_this._stopped) {
                requestAnimationFrame(tick);
            }
            else {
                _this._pending = false;
            }
        };
        requestAnimationFrame(tick);
    };
    Tick.prototype.stop = function () {
        this._stopped = true;
    };
    return Tick;
}());

function interpolateNumber(start, end) {
    var diff = end - start;
    return function (t) { return start + t * diff; };
}
function interpolateNumberArray(from, to) {
    if (from.length !== to.length) {
        throw 'two array must have the same length';
    }
    var interpolateFuncs = from.map(function (start, index) { return interpolateNumber(start, to[index]); });
    return function (t) { return interpolateFuncs.map(function (func) { return func(t); }); };
}

var compose = function () {
    var cbs = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        cbs[_i] = arguments[_i];
    }
    return function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        return cbs.reduceRight(function (prevResult, cb) { return cb(prevResult); }, args);
    };
};

function transition(option, cb) {
    var _a = option !== null && option !== void 0 ? option : {}, from = _a.from, to = _a.to, _b = _a.ease, ease = _b === void 0 ? easeLinear : _b, duration = _a.duration;
    if (from instanceof Array && !(to instanceof Array)) {
        throw 'Field from and to must have the same type.';
    }
    var interpolateFunc = from instanceof Array
        ? interpolateNumberArray(from, to)
        : interpolateNumber(from, to);
    var tick = new Tick(function (t) { return compose(cb, interpolateFunc, ease)(t); });
    tick.start(duration);
    return function () { return tick.stop(); };
}

function pie(option) {
    var width = option.width, height = option.height, data = option.data;
    var svg = ElementBuilder.create('svg');
    svg.setAttrs({
        width: width,
        height: height,
        viewBox: '0 0 100 100',
    });
    svg.style('overflow', 'visible');
    var g = ElementBuilder.create('g');
    g.setAttrs({ transform: 'translate(50, 50)' });
    svg.append(g);
    transition({
        from: 0,
        to: Math.PI * 2,
        ease: easeIn,
        duration: 500,
    }, function (totalRad) {
        var _a;
        var paths = render(data, totalRad);
        (_a = g.empty()).append.apply(_a, paths);
    });
    return svg.element;
}
function render(data, totalRad) {
    if (data === void 0) { data = []; }
    var startRad = -Math.PI / 2;
    var total = data.reduce(function (sum, _a) {
        var value = _a.value;
        return sum += value;
    }, 0);
    var paths = [];
    data.forEach(function (item) {
        if (!item.fillColor) {
            item.fillColor = randomRgbColor();
        }
        var value = item.value, strokeColor = item.strokeColor, fillColor = item.fillColor, _a = item.strokeWidth, strokeWidth = _a === void 0 ? 0 : _a;
        var fan = ElementBuilder.create('path');
        var path = new Path();
        path.moveTo(0, 0);
        var rad = value / total * totalRad;
        path.lineTo.apply(path, pointOnCircle(0, 0, 50, startRad));
        startRad += rad;
        path.ellipticalArcTo.apply(path, __spreadArray([50, 50, 0, rad > Math.PI, true], pointOnCircle(0, 0, 50, startRad)));
        path.closePath();
        fan.setAttrs({
            d: path.pathString,
            stroke: strokeColor || fillColor,
            fill: fillColor,
            'stroke-width': strokeWidth,
        });
        paths.push(fan);
    });
    return paths;
}

export { pie as default };

==> charts-pie-1.0.4.tgz: package/lib/pie.js <==
/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

function __spreadArray(to, from) {
    for (var i = 0, il = from.length, j = to.length; i < il; i++, j++)
        to[j] = from[i];
    return to;
}

var ElementBuilder = /** @class */ (function () {
    function ElementBuilder(label) {
        this._data = {};
        this._elem = createElement(label);
    }
    ElementBuilder.prototype.setData = function (data) {
        this._data = __assign(__assign({}, this._data), data);
    };
    ElementBuilder.prototype.getData = function (key) {
        var _a;
        return (_a = this._data[key]) !== null && _a !== void 0 ? _a : null;
    };
    ElementBuilder.prototype.setAttrs = function (attrs) {
        for (var _i = 0, _a = Object.entries(attrs); _i < _a.length; _i++) {
            var _b = _a[_i], key = _b[0], val = _b[1];
            this.attr(key, val);
        }
    };
    ElementBuilder.prototype.attr = function (name, val) {
        if (val) {
            setAttribute(this._elem, name, val);
            return this;
        }
        return getAttribute(this._elem, name) || '';
    };
    ElementBuilder.prototype.style = function (name, value) {
        this._elem.style[name] = value;
        return this;
    };
    Object.defineProperty(ElementBuilder.prototype, "element", {
        get: function () {
            return this._elem;
        },
        enumerable: false,
        configurable: true
    });
    ElementBuilder.prototype.append = function () {
        var _a;
        var elems = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            elems[_i] = arguments[_i];
        }
        (_a = this._elem).append.apply(_a, elems.map(function (elem) { return elem._elem; }));
        return this;
    };
    ElementBuilder.prototype.empty = function () {
        this._elem.innerHTML = '';
        return this;
    };
    ElementBuilder.prototype.addListener = function (name, cb) {
        this._elem.addEventListener(name, cb);
    };
    ElementBuilder.create = function (label, attrs) {
        var instance = new ElementBuilder(label);
        if (attrs) {
            instance.setAttrs(attrs);
        }
        return instance;
    };
    return ElementBuilder;
}());
var SVG_NAMESPACE = 'http://www.w3.org/2000/svg';
function createElement(tag) {
    return document.createElementNS(SVG_NAMESPACE, tag);
}
function setAttribute(element, name, value) {
    if (name === 'xlink:href') {
        element.setAttributeNS(SVG_NAMESPACE, name, value);
    }
    element.setAttribute(name, value);
}
function getAttribute(element, name) {
    return element.getAttribute(name);
}

var random = function (min, max) { return min + Math.floor(Math.random() * (max - min)); };
var randomRgbColor = function () {
    var rgb = Array(3).fill(0).map(function () { return random(0, 256); });
    return "rgb(" + rgb.join(',') + ")";
};

var easeLinear = function (t) { return t; };
var easeIn = function (t, order) {
    if (order === void 0) { order = 2; }
    return Math.pow(t, order);
};

var Path = /** @class */ (function () {
    function Path() {
        this._pathSegments = [];
        this._isClosed = false;
    }
    Path.prototype.addPathSegments = function (instrument) {
        var params = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            params[_i - 1] = arguments[_i];
        }
        if (this._isClosed) {
            throw 'This path is already closed';
        }
        this._pathSegments.push(__spreadArray([instrument], params));
        if (instrument === 'Z') {
            this._isClosed = true;
        }
    };
    Path.prototype.moveTo = function (x, y) {
        this.addPathSegments('M', x, y);
        return this;
    };
    Path.prototype.moveToRelative = function (dx, dy) {
        this.addPathSegments('m', dx, dy);
        return this;
    };
    Path.prototype.lineTo = function (x, y) {
        this.addPathSegments('L', x, y);
        return this;
    };
    Path.prototype.lineToRelative = function (dx, dy) {
        this.addPathSegments('l', dx, dy);
        return this;
    };
    Path.prototype.horizontalLineTo = function (x) {
        this.addPathSegments('H', x);
        return this;
    };
    Path.prototype.horizontalLineToRelative = function (dx) {
        this.addPathSegments('h', dx);
        return this;
    };
    Path.prototype.verticalLineTo = function (y) {
        this.addPathSegments('V', y);
        return this;
    };
    Path.prototype.verticalLineToRelative = function (dy) {
        this.addPathSegments('v', dy);
        return this;
    };
    Path.prototype.curveTo = function (cx1, cy1, cx2, cy2, tx, ty) {
        this.addPathSegments('C', cx1, cy1, cx2, cy2, tx, ty);
        return this;
    };
    Path.prototype.curveToRelative = function (cdx1, cdy1, cdx2, cdy2, tdx, tdy) {
        this.addPathSegments('c', cdx1, cdy1, cdx2, cdy2, tdx, tdy);
        return this;
    };
    Path.prototype.smoothCurveTo = function (cx2, cy2, tx, ty) {
        this.addPathSegments('S', cx2, cy2, tx, ty);
        return this;
    };
    Path.prototype.smoothCurveToRelative = function (cdx2, cdy2, tdx, tdy) {
        this.addPathSegments('s', cdx2, cdy2, tdx, tdy);
        return this;
    };
    Path.prototype.quadCurveTo = function (cx, cy, tx, ty) {
        this.addPathSegments('Q', cx, cy, tx, ty);
        return this;
    };
    Path.prototype.quadCurveToRelative = function (cdx, cdy, tdx, tdy) {
        this.addPathSegments('q', cdx, cdy, tdx, tdy);
        return this;
    };
    Path.prototype.smoothQuadCurveTo = function (tx, ty) {
        this.addPathSegments('T', tx, ty);
        return this;
    };
    Path.prototype.smoothQuadCurveToRelative = function (tdx, tdy) {
        this.addPathSegments('t', tdx, tdy);
        return this;
    };
    Path.prototype.ellipticalArcTo = function (rx, ry, xAxisRotation, isLargeArc, isClockWise, tx, ty) {
        this.addPathSegments('A', rx, ry, xAxisRotation, isLargeArc ? 1 : 0, isClockWise ? 1 : 0, tx, ty);
        return this;
    };
    Path.prototype.closePath = function () {
        this.addPathSegments('Z');
        return this;
    };
    Path.prototype.back = function () {
        this._pathSegments.pop();
        return this;
    };
    Path.prototype.remove = function (index) {
        this._pathSegments.splice(index, 1);
        return this;
    };
    Path.prototype.toString = function () {
        return this.pathString;
    };
    Object.defineProperty(Path.prototype, "pathString", {
        get: function () {
            return this._pathSegments.map(function (_a) {
                var ins = _a[0], params = _a.slice(1);
                return ins + params.join(' ');
            }).join(' ');
        },
        enumerable: false,
        configurable: true
    });
    Path.prototype.append = function (path) {
        if (this._isClosed) {
            throw 'This path is already closed';
        }
        this._pathSegments = __spreadArray(__spreadArray([], this._pathSegments), path._pathSegments);
        if (this._pathSegments[this._pathSegments.length - 1][0] === 'Z') {
            this._isClosed = true;
        }
        return this;
    };
    return Path;
}());

var pointOnCircle = function (cx, cy, r, rad) {
    return [
        cx + r * Math.cos(rad),
        cy + r * Math.sin(rad)
    ];
};

var Tick = /** @class */ (function () {
    function Tick(cb) {
        this._startTime = 0;
        this._duration = 0;
        this._stopped = false;
        this._pending = false;
        this._onEndCallback = null;
        this._tickCallback = cb;
    }
    Tick.prototype.start = function (duration) {
        if (this._pending)
            return;
        this._duration = duration;
        this._startTime = Date.now();
        this.tickLoop();
    };
    Tick.prototype.tickLoop = function () {
        var _this = this;
        this._pending = true;
        var tick = function () {
            var percentageComplete = (Date.now() - _this._startTime) / _this._duration;
            var over = false;
            if (percentageComplete > 1) {
                percentageComplete = 1;
                over = true;
            }
            if (!_this._stopped) {
                _this._tickCallback(percentageComplete);
            }
            if (!over && !_this._stopped) {
                requestAnimationFrame(tick);
            }
            else {
                _this._pending = false;
            }
            if (over && _this._onEndCallback) {
                _this._onEndCallback();
            }
        };
        requestAnimationFrame(tick);
    };
    Tick.prototype.stop = function () {
        this._stopped = true;
    };
    Tick.prototype.onEnd = function (cb) {
        if (cb) {
            this._onEndCallback = cb;
        }
    };
    return Tick;
}());

function interpolateNumber(start, end) {
    var diff = end - start;
    return function (t) { return start + t * diff; };
}
function interpolateNumberArray(from, to) {
    if (from.length !== to.length) {
        throw 'two array must have the same length';
    }
    var interpolateFuncs = from.map(function (start, index) { return interpolateNumber(start, to[index]); });
    return function (t) { return interpolateFuncs.map(function (func) { return func(t); }); };
}

var compose = function () {
    var cbs = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        cbs[_i] = arguments[_i];
    }
    return function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        return cbs.reduceRight(function (prevResult, cb) { return cb(prevResult); }, args);
    };
};

function transition(option, cb, onEnd) {
    var _a = option !== null && option !== void 0 ? option : {}, from = _a.from, to = _a.to, _b = _a.ease, ease = _b === void 0 ? easeLinear : _b, duration = _a.duration;
    if (from instanceof Array && !(to instanceof Array)) {
        throw 'Field from and to must have the same type.';
    }
    var interpolateFunc = from instanceof Array
        ? interpolateNumberArray(from, to)
        : interpolateNumber(from, to);
    var tick = new Tick(function (t) { return compose(cb, interpolateFunc, ease)(t); });
    tick.start(duration);
    tick.onEnd(onEnd);
    return function () { return tick.stop(); };
}

function pie(option) {
    var width = option.width, height = option.height, _a = option.focus, focus = _a === void 0 ? true : _a, data = option.data;
    var svg = ElementBuilder.create('svg');
    svg.setAttrs({
        width: width,
        height: height,
        viewBox: '0 0 100 100',
    });
    svg.style('overflow', 'visible');
    var g = ElementBuilder.create('g');
    g.setAttrs({ transform: 'translate(50, 50)' });
    svg.append(g);
    var isFirstRendering = true;
    transition({
        from: 0,
        to: Math.PI * 2,
        ease: easeIn,
        duration: 500,
    }, function (totalRad) {
        var _a;
        var paths = render(data, totalRad);
        if (isFirstRendering) {
            isFirstRendering = false;
            (_a = g.empty()).append.apply(_a, paths);
            if (focus) {
                bindFocusEvents(paths);
            }
        }
    });
    return svg.element;
}
var cachedPaths = [];
function render(data, totalRad) {
    if (data === void 0) { data = []; }
    var startRad = -Math.PI / 2;
    var total = data.reduce(function (sum, _a) {
        var value = _a.value;
        return sum += value;
    }, 0);
    var paths = [];
    data.forEach(function (item, i) {
        if (!item.fillColor) {
            item.fillColor = randomRgbColor();
        }
        var value = item.value, strokeColor = item.strokeColor, fillColor = item.fillColor, _a = item.strokeWidth, strokeWidth = _a === void 0 ? 0 : _a;
        var fan = cachedPaths[i] || ElementBuilder.create('path');
        var path = new Path();
        path.moveTo(0, 0);
        var rad = value / total * totalRad;
        path.lineTo.apply(path, pointOnCircle(0, 0, 50, startRad));
        startRad += rad;
        path.ellipticalArcTo.apply(path, __spreadArray([50, 50, 0, rad > Math.PI, true], pointOnCircle(0, 0, 50, startRad)));
        path.closePath();
        fan.setAttrs({
            d: path.pathString,
            stroke: strokeColor || fillColor,
            fill: fillColor,
            'stroke-width': strokeWidth
        });
        fan.setData({
            centerRad: startRad - rad / 2,
        });
        paths.push(fan);
    });
    cachedPaths = paths;
    return paths;
}
function bindFocusEvents(paths) {
    paths.forEach(function (path) {
        path.style('transition', 'transform ease-out .25s');
        path.addListener('mouseover', function () {
            var centerRad = path.getData('centerRad');
            var center = pointOnCircle(0, 0, 5, centerRad);
            path.setAttrs({
                transform: "scale(1.2) translate(" + center.join(',') + ")"
            });
        });
        path.addListener('mouseout', function () {
            path.setAttrs({
                transform: 'translate(0, 0)'
            });
        });
    });
}

export { ElementBuilder, Path, Tick, pie as default, pointOnCircle, transition };
